#!/usr/bin/perl

use lib '.';
use Patch;


# Process command line args
$xmlfile  = shift;

# Define some useful constants
$ver_file1 = "/etc/lefthand/.version";
$ver_file2 = "/etc/lefthand/system/.version";

my @patch_requires_array  = ( );    # initialize patch requires array
my @patch_conflicts_array = ( );    # initialize patch clicts array
my @platform_array        = ( );    # initialize platform array
my @saniq_version_array   = ( );    # initialize support saniq versions array

# Get SAN/IQ version
if(-e $ver_file1) {
    open( VERFILE, "<$ver_file1") || die "Couldn't open file: $ver_file1\n";
}
else {
    open( VERFILE, "<$ver_file2") || die "Couldn't open file: $ver_file2\n";
}

while(<VERFILE>) 
{
    if( /(\d+\.\d+\.\d+\.\d+)/ )
    {
        $nsm_version = $1;
    }
}
close (VERFILE);

# Parse the xml file
open( XMLFILE, "<$xmlfile") || die "Couldn't open file: $xmlfile\n"; 
while(<XMLFILE>) 
{
    if( /<SANIQ_VERSION>\s*(\d+\.\d+\.\d+\.\d+)\s*<\/SANIQ_VERSION>/ ) {
        push @saniq_version_array, $1;
    }
    elsif( /<SANIQ_VERSION>(ALL)<\/SANIQ_VERSION>/ ) {
        push @saniq_version_array, $1;
    }
    elsif( /<PATCH_NUMBER>(\w+-\w+)<\/PATCH_NUMBER>/ ) {
        # Extract patch number
        $patch_number = $1;
    }
    elsif( /<PATCH_TYPE>(\w+)<\/PATCH_TYPE>/ ) {
        # Extract to patch type
        $patch_type = $1;
    }
    elsif( /<PATCH_REQUIRES>(\w+-\w+)<\/PATCH_REQUIRES>/ ) {
        # Add to required hash
        push @patch_requires_array, $1;
    }
    elsif( /<PATCH_CONFLICTS>(\w+-\w+)<\/PATCH_CONFLICTS>/ ) {
        # Add to conflicts hash
        push @patch_conflicts_array, $1;
    }
    elsif( /<PLATFORM>(.*)<\/PLATFORM>/ ) {
        # Add to platform hash
         push @platform_array,  trim($1);
    }
    elsif( /<REQUIRES_REBOOT>(\w+)<\/REQUIRES_REBOOT>/ ) {
        # Extract to requires reboot
        $requires_reboot = $1;
    }
    elsif( /<BACKOUT_AVAILABLE>(\w+)<\/BACKOUT_AVAILABLE>/ ) {
        # Extract to requires reboot
        $requires_reboot = $1;
    }
}
close(XMLFILE) ;

$valid_saniq_version = 0;

if($patch_type ne "HW") 
{
    foreach $element(@saniq_version_array)
    {
        if(($element eq $nsm_version) or ($element eq "ALL"))
        {
            $valid_saniq_version = 1;
        }
    }

    if($valid_saniq_version == 0) 
    {
        print "ERROR: This patch is NOT for SAN/IQ version $nsm_version!\n";
        exit 1;
    }
}

# Check that this patch has not been applied
if(Patch->exists($patch_number))
{
    print "NOTE: This patch has already been applied, reapplying.\n";
}

# Check that the required patches have been applied
my $element;
foreach $element(@patch_requires_array)
{
    if(! Patch->exists($element))
    {
        print "ERROR: This patch requires patch number $element to be installed first!\n";
        exit 1;
    }
}
        
# Check that the conflicting patches have not been applied
foreach $element(@patch_conflicts_array)
{
    if( Patch->exists($element))
    {
        print "ERROR: This patch can not be installed with patch $element!\n";
        exit 1;
    }
}
    
# Check that this is a valid platform
$valid_platform = 0;
$_ = `hwid --name`;
$platform_name = $_;

foreach $element(@platform_array)
{
    if(($element eq "ALL") or ($element eq "All") or ($element eq "all"))
    {
        $valid_platform = 1;
    }

    if($element eq $platform_name)
    {
        $valid_platform = 1;
    }
}

if($valid_platform == 0) 
{
    print "ERROR: This patch is not meant for the $platform_name platform\n";
    exit 1;
}

###
# _trim
# trim off beginning/ending whitespace
###
sub trim {
    my $string = shift;

    $string =~ s/^\s+//;
    $string =~ s/\s+$//;

    return $string;
}


0;

