#!/usr/bin/python

"""
NAME
  python get_bios_revision.py [test]
  
DESCRIPTION
  Check the BIOS version returned by dmidecode.

  If below minimum required version, script returns -1
  If at or newer than delivered version, script returns 4

  If test option passed, the release date returned by dmidecode is overriden
  with a date earlier than the minimum required version in effort to unit test
  that logic.
  
AUTHOR
  Erick P Bodine <erick.bodine@hp.com>
"""

import os
import re
import sys

TEST = False

def version_check(rdate):
    """ Check the current version of the BIOS vs. what we are delivering. """
    # Check to see if the current version is more up-to-date than the one that
    # we are delivering.
    #
    # Return codes:
    #   0  => we need to upgrade.
    #   4  => the current version is at or more recent than what we are
    #         delivering and we don't need to upgrade.
    
    bios_version_rc = 0
    cmth, cday, cyr = (5, 5, 2011) # The version we are delivering.
    r_mth, r_day, r_yr = [ int(token) for token in rdate.split('/') ]

    if r_yr < cyr:
        pass
    
    elif r_yr == cyr:
        if r_mth < cmth:
            pass

        elif r_mth == cmth:
            if r_day < cday:
                pass
                                
            elif r_day >= cday:
                print "Current BIOS revision equal to or newer than " \
                      +"firmware patch version [05/05/2011]"
                bios_version_rc = 4
                
            elif r_day > cday:
                print "Current BIOS revision equal to or newer than " \
                      +"firmware patch version [05/05/2011]"
                bios_version_rc = 4
                
        elif r_mth > cmth:
            # yes, I know there is no 13th mth but cmth,cday,cyr could change.
            print "Current BIOS revision equal to or newer than " \
                  +"firmware patch version [05/05/2011]"
            bios_version_rc = 4

    elif r_yr > cyr:
        print "Current BIOS revision equal to or newer than " \
              +"firmware patch version [05/05/2011]"
        bios_version_rc = 4

    return bios_version_rc


def main():
    """ The main script function. """

    # Get the output from dmidecode.
    out_file = "/tmp/dmidecode_bios_version.out"
    rc = os.system("/usr/sbin/dmidecode > %s" % out_file)
    if rc != 0:
        print "Problem running dmidecode [rc=%s] [output=%s]" % (rc, out_file)
        return rc

    # Get the BIOS release date from the dmidecode output
    found = False
    rdate = ""
    bios_re = re.compile("BIOS Information")
    rdate_re = re.compile("Release Date:(.*$)")
        
    try:
        for line in open(out_file):
            if bios_re.search(line):
                found = True
                continue

            if not found: continue
            robj = rdate_re.search(line)
            if robj:
                rdate = robj.group(1).strip()
                print "Found BIOS release date: %s" % rdate
                break
        
    except (IOError, OSError), ios_error:
        print "Problem getting BIOS release date!! [%s - output=%s]" % \
              (str(ios_error), out_file)
        return -1

    if TEST:
        rdate = "5/23/2011"
        print "Testing more recent BIOS version... [%s]" % rdate
        
    vc_rc = version_check(rdate)
    print "version_check rc [%d]" % vc_rc

    return vc_rc

    
if __name__ == '__main__':
    try:
        TEST = sys.argv[1]
    except IndexError:
        pass
    
    sys.exit(main())
    
        
